/**************************************************************************
*
*	CommsMessage.h
*
*	Contains a class which holds a message passed between the server and
*	the client
*
*	Author: Saxon Druce
*
*	Copyright (c) 2004 Block Software Pty Ltd
*
**************************************************************************/

#ifndef _COMMSMESSAGE_H
#define _COMMSMESSAGE_H

/*************************************************************************/
// Include files
/*************************************************************************/

#include <vector>

/*************************************************************************/
// Defines
/*************************************************************************/

/*************************************************************************/
// Public types
/*************************************************************************/

// CommsMessage_t class - the class which contains a message
class CommsMessage_t
{
public:

	// Constructor
	CommsMessage_t();

	// Destructor
	~CommsMessage_t();

	// Clear() - Clears any content in the message.
	void Clear();

	// AddInt() - Adds the specified integer value to the message.
	void AddInt(int Value);

	// AddDouble() - Adds the specified double value to the message.
	void AddDouble(double Value);

	// AddString() - Adds the specified string value to the message.
	void AddString(const std::string& Value);

	// GetInt() - Reads the next integer value from the message.
	// Throws an exception if the next value isn't an integer.
	int GetInt();

	// GetDouble() - Reads the next double value from the message.
	// Throws an exception if the next value isn't a double.
	double GetDouble();

	// GetString() - Reads the next string value from the message.
	// Throws an exception if the next value isn't a string.
	std::string GetString();

	// GetInternalData() - Returns a reference to the message's internal
	// data.
	const std::vector<char>& GetInternalData() const;

	// SetInternalData() - Sets the message's internal data to the
	// data in the given range (Start <= [data] < End).
	void SetInternalData(
		std::vector<char>::iterator Start,
		std::vector<char>::iterator End);

private:

	// possible types of values in a message
	enum ItemType_e
	{
		eInteger = 1,
		eDouble = 2,
		eString = 3,
	};

	// Note that the default copy constructor and assignment operator are 
	// correct, because the following two types manage this correctly

	// the data of the message
	std::vector<char> m_Data;

	// the position which the next read from the message will start at
	size_t m_ReadPosition;
};

/*************************************************************************/

#endif // _COMMSMESSAGE_H
