/**************************************************************************
*
*	CriticalSection.h
*
*	A critical section helper class
*
*	Author: Saxon Druce
*
*	Copyright (c) 2004 Block Software Pty Ltd
*
**************************************************************************/

#ifndef _CRITICALSECTION_H
#define _CRITICALSECTION_H

/*************************************************************************/
// Include files
/*************************************************************************/

#include <windows.h>

/*************************************************************************/
// Defines
/*************************************************************************/

/*************************************************************************/
// Public types
/*************************************************************************/

// CriticalSection_t class - basic critical section wrapper
class CriticalSection_t
{
public:

	CriticalSection_t() { InitializeCriticalSection(&m_CriticalSection); }
	~CriticalSection_t() { DeleteCriticalSection(&m_CriticalSection); }

	void Lock() { EnterCriticalSection(&m_CriticalSection); }
	void Unlock() { LeaveCriticalSection(&m_CriticalSection); }

private:

	CRITICAL_SECTION m_CriticalSection;
};

// Lock_t class - basic critical section locker
class Lock_t
{
public:

	Lock_t(CriticalSection_t& CriticalSection) : m_CriticalSection(CriticalSection)
	{
		m_CriticalSection.Lock();
	}

	~Lock_t()
	{
		m_CriticalSection.Unlock();
	}

private:

	CriticalSection_t& m_CriticalSection;
};

/*************************************************************************/

#endif // _CRITICALSECTION_H
